/*
 * ARM64 kexec support.
 */

#define _GNU_SOURCE

#include <errno.h>
#include <stddef.h>

//#include <linux/kexec.h>

#include "../../kexec.h"
#include "../../kexec-syscall.h"
#include "kexec-arm64.h"


void arch_usage(void)
{
	fprintf(stderr, "%s:%d: ->\n", __func__, __LINE__);

	printf(
"     --append=STRING       Set the kernel command line to STRING.\n"
"     --command-line=STRING Set the kernel command line to STRING.\n"
"     --dtb=FILE            Use FILE as the device tree blob.\n"
"     --initrd=FILE         Use FILE as the kernel initial ramdisk.\n"
"     --ramdisk=FILE        Use FILE as the kernel initial ramdisk.\n");

	fprintf(stderr, "%s:%d: <-\n", __func__, __LINE__);
}

int arch_process_options(int UNUSED(argc), char **UNUSED(argv))
{
	fprintf(stderr, "%s:%d: do\n", __func__, __LINE__);
	return 0;
}

const struct arch_map_entry arches[] = {
	{ "aarch64", KEXEC_ARCH_ARM64 },
	{ NULL, 0 },
};

void arch_update_purgatory(struct kexec_info *UNUSED(info))
{
	fprintf(stderr, "%s:%d: do\n", __func__, __LINE__);
}

unsigned long virt_to_phys(unsigned long addr)
{
	fprintf(stderr, "%s:%d: %016lx -> %016lx\n", __func__, __LINE__, addr,
		addr + 0x080000000UL);
	return addr + 0x080000000UL;
}

void add_segment(struct kexec_info *info, const void *buf, size_t bufsz,
	unsigned long base, size_t memsz)
{
	fprintf(stderr, "%s:%d: ->\n", __func__, __LINE__);
	add_segment_phys_virt(info, buf, bufsz, base, memsz, 1);
	fprintf(stderr, "%s:%d: <-\n", __func__, __LINE__);
}

static int get_memory_ranges_1(struct memory_range **range, int *ranges,
			unsigned long kexec_flags)
{
	static struct memory_range memory_range[KEXEC_SEGMENT_MAX];
	const char *iomem;
	int range_count = 0;
	char line[MAX_LINE];
	FILE *fp;

	iomem = proc_iomem();
	fp = fopen(iomem, "r");

	if (!fp) {
		fprintf(stderr, "Cannot open %s: %s\n",
			iomem, strerror(errno));
		return -1;
	}

	dbgprintf("memory ranges:\n");

	while(fgets(line, sizeof(line), fp) != 0) {
		struct memory_range r;
		char *str;
		int consumed;

		if (range_count >= KEXEC_SEGMENT_MAX)
			break;

		if (sscanf(line, "%Lx-%Lx : %n", &r.start, &r.end, &consumed)
			!= 2)
			continue;

		str = line + consumed;
		r.end++;

		if (memcmp(str, "System RAM\n", 11)) {
			dbgprintf(" Skip: %016Lx - %016Lx : %s", r.start, r.end,
				str);
			continue;
		}

		r.type = RANGE_RAM;
		memory_range[range_count] = r;
		range_count++;

		dbgprintf(" Add:  %016Lx - %016Lx : %s", r.start, r.end, str);
	}

	fclose(fp);
	*range = memory_range;
	*ranges = range_count;

	return 0;
}

static int get_memory_ranges_2(struct memory_range **range, int *ranges,
		      unsigned long UNUSED(kexec_flags))
{
	static struct memory_range memory_range[2];

	memory_range[0].start = 0x080000000;
	memory_range[0].end   = 0x100000000;
	memory_range[0].type = RANGE_RAM;

	memory_range[1].start = 0x900000000;
	memory_range[1].end   = 0x880000000;
	memory_range[1].type = RANGE_RAM;

	*range = memory_range;
	*ranges = sizeof(memory_range) / sizeof(memory_range[0]);

	return 0;
}

int get_memory_ranges(struct memory_range **range, int *ranges,
			unsigned long kexec_flags)
{
	/* FIXME: Should get this info from device tree. */
	
	return get_memory_ranges_1(range, ranges, kexec_flags);
}

struct file_type file_type[] = {
	{ "elf-arm64", elf_arm64_probe, elf_arm64_load, elf_arm64_usage },
};

int file_types = sizeof(file_type) / sizeof(file_type[0]);

int arch_compat_trampoline(struct kexec_info *info)
{
	fprintf(stderr, "%s:%d: do\n", __func__, __LINE__);
	return 0;
}

void arch_reuse_initrd(void)
{
}

int machine_verify_elf_rel(struct mem_ehdr *ehdr)
{
	(void)ehdr;

	fprintf(stderr, "%s:%d: do\n", __func__, __LINE__);
	return 0;
}

void machine_apply_elf_rel(struct mem_ehdr *ehdr, unsigned long r_type,
	void *location, unsigned long address, unsigned long value)
{
	(void)ehdr;
	(void)r_type;
	(void)location;
	(void)address;
	(void)value;
	fprintf(stderr, "%s:%d: do\n", __func__, __LINE__);
}
