/*
 * ARM64 kexec support.
 */

#define _GNU_SOURCE

#include <elf.h>
#include <getopt.h>

#include "../../kexec-syscall.h"

#include "kexec-arm64.h"
#include "arch/options.h"

#if !defined(EM_AARCH64)
# define EM_AARCH64 183
#endif

int elf_arm64_probe(const char *buf, off_t len)
{
	int result;
	struct mem_ehdr ehdr;

	fprintf(stderr, "%s:%d: ->\n", __func__, __LINE__);
	
	result = build_elf_exec_info(buf, len, &ehdr, 0);

	if (result < 0) {
		dbgprintf("Not an ELF executable\n");
		goto out;
	}

	if (ehdr.e_machine != EM_AARCH64) {
		dbgprintf("Not an AARCH64 executable\n");
		result = -1;
		goto out;
	}

	result = 0;

out:
	free_elf_info(&ehdr);
	fprintf(stderr, "%s:%d: <-\n", __func__, __LINE__);
	return result;
}

int elf_arm64_load(int argc, char **argv, const char *buf, off_t len,
			struct kexec_info *info)
{
	static const struct option options[] = {
		KEXEC_ARCH_OPTIONS
		{ 0 }
	};
	static const char short_options[] = KEXEC_OPT_STR "";
	const char *command_line = NULL;
	unsigned int command_line_len = 0;
	const char *ramdisk = NULL;
	const char *dtb = NULL;
	int opt;
	struct mem_ehdr ehdr;
	int result;

	fprintf(stderr, "%s:%d: ->\n", __func__, __LINE__);

	while ((opt = getopt_long(argc, argv, short_options, options, 0))
		!= -1) {
		switch (opt) {
		default:
			if (opt < OPT_MAX) /* Ignore core options */
				break;
		case OPT_APPEND:
			command_line = optarg;
			command_line_len = strlen(command_line) + 1;
			break;
		case OPT_RAMDISK:
			ramdisk = optarg;
			break;
		case OPT_DTB:
			dtb = optarg;
			break;
		}
	}

	fprintf(stderr, "%s:%d: command_line: %s\n", __func__, __LINE__, command_line);
	fprintf(stderr, "%s:%d: ramdisk: %s\n", __func__, __LINE__, ramdisk);
	fprintf(stderr, "%s:%d: dtb: %s\n", __func__, __LINE__, dtb);

	if (info->kexec_flags & KEXEC_ON_CRASH) {
		fprintf(stderr, "kexec: kdump not yet supported on arm64\n");
		return -1;
	}

	result = build_elf_exec_info(buf, len, &ehdr, 0);

	if (result < 0) {
		free_elf_info(&ehdr);
		fprintf(stderr, "%s:%d: free_elf_info failed\n", __func__,
			__LINE__);
		return result;
	}

	elf_exec_build_load(info, &ehdr, buf, len, 0);

	info->entry = (void*)0x80080000UL;  // FIXME

	fprintf(stderr, "%s:%d: <-\n", __func__, __LINE__);
	return 0;
}

void elf_arm64_usage(void)
{
	fprintf(stderr, "%s:%d: ->\n", __func__, __LINE__);
	fprintf(stderr, "%s:%d: <-\n", __func__, __LINE__);
}
